{% extends "organizations_base.html" %}
{% load i18n %}
{% block content %}
  <h1 id="login-header">Log in to your dashboard</h1>
  <div id="login-box">
    <form method="post" action="{% url login %}">{% csrf_token %}
        {{ form }}
        <input type="submit" value="Login">
    </form>
	<p>{% trans "Forgotten your password?" %} <a href="{% url "password_reset" %}">{% trans "Reset it" %}.</a></p>
  </div>
{% endblock %}

